module ad7928_spi(
    clk         ,
    rst_n       ,
    out_cs      ,
    out_data    ,
    out_sclk    ,
    ad_cs       ,
    ad_sclk     ,
    ad_din      ,
    ad_dout       
);

parameter        WRITE = 1'b1;
parameter        PM    = 2'b11;
parameter        SEQ   = 1'b0;
parameter        SHADOW= 1'b0;
parameter        RANGE = 1'b0;
parameter        CODING= 1'b1;

input         clk        ;
input         rst_n      ;
output        out_sclk   ;  
output        out_cs     ;
output        out_data   ;

input         ad_dout    ;  
output        ad_din     ;  
output        ad_sclk    ;
output        ad_cs      ;

reg           ad_din     ;  
reg           ad_cs      ;
reg           ad_sclk    ;

reg   [  0:0] cnt0       ;
wire          add_cnt0   ;
wire          end_cnt0   ;

reg   [ 7:0]  cnt1       ;
wire          add_cnt1   ;
wire          end_cnt1   ;

reg   [  0:0] cnt2       ;
wire          add_cnt2   ;
wire          end_cnt2   ;

always @(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        cnt0 <= 0;
    end
    else if(add_cnt0)begin
        if(end_cnt0)
            cnt0 <= 0;
        else
            cnt0 <= cnt0 + 1;
    end
end

assign add_cnt0 = 1;
assign end_cnt0 = add_cnt0 && cnt0==2-1 ;

always @(posedge clk or negedge rst_n)begin 
    if(!rst_n)begin
        cnt1 <= 0;
    end
    else if(add_cnt1)begin
        if(end_cnt1)
            cnt1 <= 0;
        else
            cnt1 <= cnt1 + 1;
    end
end

assign add_cnt1 = end_cnt0;
assign end_cnt1 = add_cnt1 && cnt1==100-1 ;

always @(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        cnt2 <= 0;
    end
    else if(add_cnt2)begin
        if(end_cnt2)
            cnt2 <= 0;
        else
            cnt2 <= cnt2 + 1;
    end
end

assign add_cnt2 = end_cnt1;
assign end_cnt2 = add_cnt2 && cnt2==2-1 ;
 
assign middle = add_cnt0 && cnt0==1-1;

assign sclk_en0 = middle && cnt1>=1 && cnt1 <17;

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        ad_sclk <= 1;
    end
    else if(sclk_en0)begin
        ad_sclk <= 0;
    end
    else if(end_cnt0)begin
        ad_sclk <= 1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        ad_cs <= 1;
    end
    else if(middle && cnt1==0)begin
        ad_cs <= 0;
    end
    else if(middle && cnt1==17 )begin
        ad_cs <= 1;
    end
end

wire[15:0]   data;
always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        ad_din <= 0;
    end
    else if(end_cnt0 && cnt1<16)begin
        ad_din <= data[15-cnt1];
    end
end

assign data = {WRITE,SEQ,1'b0,cnt2,2'b00,PM,SHADOW,1'b0,RANGE,CODING,4'b0};

assign out_cs   = ad_cs   ;
assign out_data = ad_dout ;
assign out_sclk = ad_sclk ;


endmodule
